@@ -1,21 +1,18 @@
-import logging
-from logging import handlers as logging_handlers
 import os
 import psutil
-import setproctitle
 import time
 
 from prometheus_client import Gauge
 from prometheus_client import start_http_server
 
+from shakenfist.daemons import daemon
 from shakenfist import config
 from shakenfist import db
+from shakenfist import logutil
 from shakenfist import util
 
 
-LOG = logging.getLogger(__file__)
-LOG.setLevel(logging.INFO)
-LOG.addHandler(logging_handlers.SysLogHandler(address='/dev/log'))
+LOG, _ = logutil.setup(__name__)
 
 
 def _get_stats():
@@ -32,7 +29,7 @@ def _get_stats():
 
     retval['cpu_max_per_instance'] = conn.getMaxVcpus(None)
 
-    # This is disable as data we don't currently use
+    # This is disabled as data we don't currently use
     # for i in range(present_cpus):
     #    per_cpu_stats = conn.getCPUStats(i)
     #    for key in per_cpu_stats:
@@ -97,7 +94,11 @@ def _get_stats():
     total_instance_cpu_time = 0
 
     for guest in conn.listAllDomains():
-        active = guest.isActive() == 1
+        try:
+            active = guest.isActive() == 1
+        except Exception:
+            active = False
+
         _, maxmem, mem, cpus, cpu_time = guest.info()
 
         if active:
@@ -108,6 +109,10 @@ def _get_stats():
             total_instance_vcpus += cpus
             total_instance_cpu_time += cpu_time
 
+    # Queue health statistics
+    node_queue_processing, node_queue_waiting = db.get_queue_length(
+        config.parsed.get('NODE_NAME'))
+
     retval.update({
         'cpu_total_instance_vcpus': total_instance_vcpus,
         'cpu_total_instance_cpu_time': total_instance_cpu_time,
@@ -115,32 +120,64 @@ def _get_stats():
         'memory_total_instance_actual': total_instance_actual_memory // 1024,
         'instances_total': total_instances,
         'instances_active': total_active_instances,
+        'node_queue_processing': node_queue_processing,
+        'node_queue_waiting': node_queue_waiting,
     })
 
+    if util.is_network_node():
+        network_queue_processing, network_queue_waiting = db.get_queue_length(
+            'networknode')
+
+        retval.update({
+            'network_queue_processing': network_queue_processing,
+            'network_queue_waiting': network_queue_waiting,
+        })
+
     return retval
 
 
-class monitor(object):
-    def __init__(self):
-        setproctitle.setproctitle('sf resources')
+class Monitor(daemon.Daemon):
+    def __init__(self, id):
+        super(Monitor, self).__init__(id)
         start_http_server(config.parsed.get('PROMETHEUS_METRICS_PORT'))
 
     def run(self):
-        gauges = {'updated_at': Gauge(
-            'updated_at', 'The last time metrics were updated')}
+        LOG.info('Starting')
+        gauges = {
+            'updated_at': Gauge('updated_at', 'The last time metrics were updated')
+        }
+
+        last_metrics = 0
+
+        def update_metrics():
+            global last_metrics
+
+            stats = _get_stats()
+            for metric in stats:
+                if metric not in gauges:
+                    gauges[metric] = Gauge(metric, '')
+                gauges[metric].set(stats[metric])
+
+            db.update_metrics_bulk(stats)
+            LOG.info('Updated metrics')
+            gauges['updated_at'].set_to_current_time()
 
         while True:
             try:
-                stats = _get_stats()
-                for metric in stats:
-                    if metric not in gauges:
-                        gauges[metric] = Gauge(metric, '')
-                    gauges[metric].set(stats[metric])
-                db.update_metrics_bulk(stats)
+                jobname, _ = db.dequeue(
+                    '%s-metrics' % config.parsed.get('NODE_NAME'))
+                if jobname:
+                    if time.time() - last_metrics > 2:
+                        update_metrics()
+                        last_metrics = time.time()
+                    db.resolve('%s-metrics' % config.parsed.get('NODE_NAME'),
+                               jobname)
+                else:
+                    time.sleep(0.2)
 
-                gauges['updated_at'].set_to_current_time()
+                if time.time() - last_metrics > config.parsed.get('SCHEDULER_CACHE_TIMEOUT'):
+                    update_metrics()
+                    last_metrics = time.time()
 
             except Exception as e:
                 util.ignore_exception('resource statistics', e)
-
-            time.sleep(config.parsed.get('SCHEDULER_CACHE_TIMEOUT'))
