@@ -1,5 +1,3 @@
-import logging
-from logging import handlers as logging_handlers
 import multiprocessing
 import os
 import re
@@ -8,15 +6,17 @@ import signal
 import time
 
 from shakenfist import config
+from shakenfist.daemons import daemon
 from shakenfist import db
+from shakenfist import logutil
 
 
-LOG = logging.getLogger(__file__)
-LOG.setLevel(logging.INFO)
-LOG.addHandler(logging_handlers.SysLogHandler(address='/dev/log'))
+LOG, _ = logutil.setup(__name__)
 
 
 def observe(path, instance_uuid):
+    setproctitle.setproctitle(
+        '%s-%s' % (daemon.process_name('triggers'), instance_uuid))
     regexps = {
         'login prompt': ['^.* login: .*', re.compile('.* login: .*')]
     }
@@ -25,41 +25,41 @@ def observe(path, instance_uuid):
         time.sleep(1)
     fd = os.open(path, os.O_RDONLY | os.O_NONBLOCK)
 
-    LOG.info('Monitoring %s for triggers' % path)
+    log_ctx = LOG.withInstance(instance_uuid)
+    log_ctx.withField('path', path).info('Monitoring path for triggers')
     db.add_event('instance', instance_uuid, 'trigger monitor',
                  'detected console log', None, None)
-    os.lseek(fd, 0, os.SEEK_END)
+
+    # Sometimes the trigger process is slow to start, so rewind 4KB to ensure
+    # that the last few log lines are not missed. (4KB since Cloud-Init can be
+    # noisy after the login prompt.)
+    os.lseek(fd, max(0, os.fstat(fd).st_size - 4096), os.SEEK_SET)
 
     buffer = ''
     while True:
-        d = os.read(fd, 1024).decode('utf-8')
+        d = os.read(fd, 1024).decode('utf-8', errors='ignore')
         if d:
-            LOG.debug('Trigger observer read %d bytes for instance %s'
-                      % (len(d), instance_uuid))
             buffer += d
             lines = buffer.split('\n')
             buffer = lines[-1]
 
             for line in lines:
                 if line:
-                    LOG.debug('Trigger observer checks "%s" for instance %s'
-                              % (line, instance_uuid))
                     for trigger in regexps:
                         m = regexps[trigger][1].match(line)
                         if m:
-                            LOG.info('Trigger %s matched for instance %s'
-                                     % (trigger, instance_uuid))
+                            log_ctx.withField('trigger', trigger,
+                                              ).info('Trigger matched')
                             db.add_event('instance', instance_uuid, 'trigger',
                                          None, None, trigger)
-
-        time.sleep(1)
+        else:
+            # Only pause if there was no data to read
+            time.sleep(1)
 
 
-class monitor(object):
-    def __init__(self):
-        setproctitle.setproctitle('sf triggers')
-
+class Monitor(daemon.Daemon):
     def run(self):
+        LOG.info('Starting Monitor Daemon')
         observers = {}
 
         while True:
@@ -67,15 +67,18 @@ class monitor(object):
             all_observers = list(observers.keys())
             for instance_uuid in all_observers:
                 if not observers[instance_uuid].is_alive():
-                    LOG.info('Trigger observer for instance %s has terminated'
-                             % instance_uuid)
+                    # Reap process
+                    observers[instance_uuid].join(1)
+                    LOG.withInstance(instance_uuid
+                                     ).info('Trigger observer has terminated')
                     db.add_event(
                         'instance', instance_uuid, 'trigger monitor', 'crashed', None, None)
                     del observers[instance_uuid]
 
             # Start missing observers
             extra_instances = list(observers.keys())
-            for inst in list(db.get_instances(only_node=config.parsed.get('NODE_NAME'))):
+
+            for inst in db.get_instances(only_node=config.parsed.get('NODE_NAME')):
                 if inst['uuid'] in extra_instances:
                     extra_instances.remove(inst['uuid'])
 
@@ -87,12 +90,13 @@ class monitor(object):
                         config.parsed.get('STORAGE_PATH'), 'instances', inst['uuid'], 'console.log')
                     p = multiprocessing.Process(
                         target=observe, args=(console_path, inst['uuid']),
-                        name='sf trigger %s' % inst['uuid'])
+                        name='%s-%s' % (daemon.process_name('triggers'),
+                                        inst['uuid']))
                     p.start()
 
                     observers[inst['uuid']] = p
-                    LOG.info('Started trigger observer for instance %s'
-                             % inst['uuid'])
+                    LOG.withInstance(inst['uuid']).info(
+                        'Started trigger observer')
                     db.add_event(
                         'instance', inst['uuid'], 'trigger monitor', 'started', None, None)
 
@@ -105,8 +109,8 @@ class monitor(object):
                     pass
 
                 del observers[instance_uuid]
-                LOG.info('Finished trigger observer for instance %s'
-                         % instance_uuid)
+                LOG.withInstance(instance_uuid).info(
+                    'Finished trigger observer')
                 db.add_event(
                     'instance', instance_uuid, 'trigger monitor', 'finished', None, None)
 
