@@ -1,22 +1,24 @@
 # Copyright 2020 Michael Still
 
-import logging
-from logging import handlers as logging_handlers
-import randmac
 import random
+import socket
 import time
 import uuid
 
 
 from shakenfist import config
 from shakenfist import etcd
+from shakenfist import exceptions
 from shakenfist import ipmanager
+from shakenfist import logutil
 from shakenfist import util
+from shakenfist.tasks import DeleteInstanceTask, ErrorInstanceTask
+
+# TODO(andy): Change back to 5 once network bugs fixed
+ETCD_ATTEMPT_TIMEOUT = 15
 
 
-LOG = logging.getLogger(__file__)
-LOG.setLevel(logging.INFO)
-LOG.addHandler(logging_handlers.SysLogHandler(address='/dev/log'))
+LOG, _ = logutil.setup(__name__)
 
 
 def see_this_node():
@@ -32,40 +34,51 @@ def see_this_node():
         ttl=120)
 
 
-def get_lock(name, ttl=60):
-    return etcd.get_lock(name, ttl=ttl)
+def get_lock(objecttype, subtype, name, ttl=60, timeout=ETCD_ATTEMPT_TIMEOUT,
+             relatedobjects=None, log_ctx=LOG):
+    return etcd.get_lock(objecttype, subtype, name, ttl=ttl, timeout=timeout,
+                         log_ctx=log_ctx)
+
+
+def refresh_lock(lock, relatedobjects=None, log_ctx=LOG):
+    if lock:
+        etcd.refresh_lock(lock, log_ctx=log_ctx)
+
+
+def refresh_locks(locks, relatedobjects=None, log_ctx=LOG):
+    if locks:
+        for lock in locks:
+            refresh_lock(lock, log_ctx=log_ctx)
+
+
+def clear_stale_locks():
+    etcd.clear_stale_locks()
 
 
 def get_node_ips():
-    see_this_node()
     for value in etcd.get_all('node', None):
         yield value['ip']
 
 
 def get_node(fqdn):
-    see_this_node()
     return etcd.get('node', None, fqdn)
 
 
 def get_nodes():
-    see_this_node()
     return etcd.get_all('node', None)
 
 
 def get_network_node():
-    see_this_node()
     for n in get_nodes():
         if n['ip'] == config.parsed.get('NETWORK_NODE_IP'):
             return n
 
 
 def get_network(network_uuid):
-    see_this_node()
     return etcd.get('network', None, network_uuid)
 
 
 def get_networks(all=False, namespace=None):
-    see_this_node()
     for n in etcd.get_all('network', None):
         if n['uuid'] == 'floating':
             continue
@@ -80,24 +93,23 @@ def get_networks(all=False, namespace=None):
 
 def allocate_network(netblock, provide_dhcp=True, provide_nat=False, name=None,
                      namespace=None):
-    see_this_node()
 
-    netid = str(uuid.uuid4())
+    net_id = str(uuid.uuid4())
     ipm = ipmanager.NetBlock(netblock)
-    etcd.put('ipmanager', None, netid, ipm.save())
+    etcd.put('ipmanager', None, net_id, ipm.save())
 
-    with etcd.get_lock('sf/vxlan') as _:
+    with etcd.get_lock('vxlan', None, 'all'):
         vxid = 1
         while etcd.get('vxlan', None, vxid):
             vxid += 1
 
         etcd.put('vxlan', None, vxid,
                  {
-                     'network_uuid': netid
+                     'network_uuid': net_id
                  })
 
     d = {
-        'uuid': netid,
+        'uuid': net_id,
         'vxid': vxid,
         'netblock': netblock,
         'provide_dhcp': provide_dhcp,
@@ -108,12 +120,11 @@ def allocate_network(netblock, provide_dhcp=True, provide_nat=False, name=None,
         'state': 'initial',
         'state_updated': time.time()
     }
-    etcd.put('network', None, netid, d)
+    etcd.put('network', None, net_id, d)
     return d
 
 
 def update_network_state(network_uuid, state):
-    see_this_node()
     n = get_network(network_uuid)
     n['state'] = state
     n['state_updated'] = time.time()
@@ -125,7 +136,6 @@ def update_network_state(network_uuid, state):
 
 
 def get_stale_networks(delay):
-    see_this_node()
     for n in etcd.get_all('network', None):
         if n['state'] in ['deleted', 'error']:
             if time.time() - n['state_updated'] > delay:
@@ -133,14 +143,12 @@ def get_stale_networks(delay):
 
 
 def hard_delete_network(network_uuid):
-    see_this_node()
     etcd.delete('network', None, network_uuid)
     etcd.delete_all('event/network', network_uuid)
     delete_metadata('network', network_uuid)
 
 
 def create_floating_network(netblock):
-    see_this_node()
     ipm = ipmanager.NetBlock(netblock)
     etcd.put('ipmanager', None, 'floating', ipm.save())
     etcd.put('network', None, 'floating',
@@ -159,7 +167,6 @@ def create_floating_network(netblock):
 
 
 def get_ipmanager(network_uuid):
-    see_this_node()
     ipm = etcd.get('ipmanager', None, network_uuid)
     if not ipm:
         raise Exception('IP Manager not found for network %s' % network_uuid)
@@ -167,30 +174,28 @@ def get_ipmanager(network_uuid):
 
 
 def persist_ipmanager(network_uuid, data):
-    see_this_node()
     etcd.put('ipmanager', None, network_uuid, data)
 
 
 def persist_floating_gateway(network_uuid, gateway):
-    see_this_node()
     n = get_network(network_uuid)
     n['floating_gateway'] = gateway
     etcd.put('network', None, network_uuid, n)
 
 
 def get_instance(instance_uuid):
-    see_this_node()
     i = etcd.get('instance', None, instance_uuid)
     if not i:
         return None
 
-    if not 'video' in i:
+    if 'video' not in i:
         i['video'] = {'model': 'cirrus', 'memory': 16384}
+    if 'error_message' not in i:
+        i['error_message'] = None
     return i
 
 
 def get_instances(only_node=None, all=False, namespace=None):
-    see_this_node()
     for i in etcd.get_all('instance', None):
         if only_node and i['node'] != only_node:
             continue
@@ -201,22 +206,22 @@ def get_instances(only_node=None, all=False, namespace=None):
             if namespace not in [i['namespace'], 'system']:
                 continue
 
-        if not 'video' in i:
+        if 'video' not in i:
             i['video'] = {'model': 'cirrus', 'memory': 16384}
+        if 'error_message' not in i:
+            i['error_message'] = None
 
         yield i
 
 
 def persist_block_devices(instance_uuid, block_devices):
-    see_this_node()
     i = get_instance(instance_uuid)
     i['block_devices'] = block_devices
     etcd.put('instance', None, instance_uuid, i)
 
 
 def create_instance(instance_uuid, name, cpus, memory_mb, disk_spec, ssh_key,
-                    user_data, namespace, video):
-    see_this_node()
+                    user_data, namespace, video, requested_placement):
     d = {
         'uuid': instance_uuid,
         'name': name,
@@ -233,14 +238,17 @@ def create_instance(instance_uuid, name, cpus, memory_mb, disk_spec, ssh_key,
         'state_updated': time.time(),
         'namespace': namespace,
         'power_state': 'initial',
-        'video': video
+        'video': video,
+        'node_history': [],
+        'error_message': None,
+        'requested_placement': None,
+        'placement_attempts': 0,
     }
     etcd.put('instance', None, instance_uuid, d)
     return d
 
 
 def place_instance(instance_uuid, node):
-    see_this_node()
     i = get_instance(instance_uuid)
 
     # We don't write unchanged things to the database
@@ -248,18 +256,17 @@ def place_instance(instance_uuid, node):
         return
 
     i['node'] = node
+    i['placement_attempts'] = i.get('placement_attempts', 0) + 1
     etcd.put('instance', None, instance_uuid, i)
 
 
 def instance_enforced_deletes_increment(instance_uuid):
-    see_this_node()
     i = get_instance(instance_uuid)
     i['enforced_deletes'] = i.get('enforced_deletes', 0) + 1
     etcd.put('instance', None, instance_uuid, i)
 
 
 def update_instance_state(instance_uuid, state):
-    see_this_node()
     i = get_instance(instance_uuid)
 
     # We don't write unchanged things to the database
@@ -276,7 +283,6 @@ def update_instance_state(instance_uuid, state):
 
 
 def update_instance_power_state(instance_uuid, state):
-    see_this_node()
     i = get_instance(instance_uuid)
 
     # We don't write unchanged things to the database
@@ -296,15 +302,22 @@ def update_instance_power_state(instance_uuid, state):
     etcd.put('instance', None, instance_uuid, i)
 
 
+def update_instance_error_message(instance_uuid, error_message):
+    i = get_instance(instance_uuid)
+    i['error_message'] = error_message
+    etcd.put('instance', None, instance_uuid, i)
+
+    add_event('instance', instance_uuid, 'error message',
+              error_message, None, None)
+
+
 def hard_delete_instance(instance_uuid):
-    see_this_node()
     etcd.delete('instance', None, instance_uuid)
     etcd.delete_all('event/instance', instance_uuid)
     delete_metadata('instance', instance_uuid)
 
 
 def get_stale_instances(delay):
-    see_this_node()
     for i in etcd.get_all('instance', None):
         if i['state'] in ['deleted', 'error']:
             if time.time() - i['state_updated'] > delay:
@@ -312,14 +325,11 @@ def get_stale_instances(delay):
 
 
 def create_network_interface(interface_uuid, netdesc, instance_uuid, order):
-    see_this_node()
     if 'macaddress' not in netdesc or not netdesc['macaddress']:
-        with etcd.get_lock('sf/macaddress', ttl=120) as _:
-            possible_mac = str(randmac.RandMac(
-                '00:00:00:00:00:00', False)).lstrip('\'').rstrip('\'')
+        with etcd.get_lock('macaddress', None, 'all', ttl=120):
+            possible_mac = util.random_macaddr()
             while etcd.get('macaddress', None, possible_mac):
-                possible_mac = str(randmac.RandMac(
-                    '00:00:00:00:00:00', False)).lstrip('\'').rstrip('\'')
+                possible_mac = util.random_macaddr()
 
             etcd.put('macaddress', None, possible_mac,
                      {
@@ -343,7 +353,6 @@ def create_network_interface(interface_uuid, netdesc, instance_uuid, order):
 
 
 def get_stale_network_interfaces(delay):
-    see_this_node()
     for n in etcd.get_all('networkinterface', None):
         if n['state'] in ['deleted', 'error']:
             if time.time() - n['state_updated'] > delay:
@@ -351,13 +360,11 @@ def get_stale_network_interfaces(delay):
 
 
 def hard_delete_network_interface(interface_uuid):
-    see_this_node()
     etcd.delete('networkinterface', None, interface_uuid)
     etcd.delete_all('event/networkinterface', interface_uuid)
 
 
 def get_instance_interfaces(instance_uuid):
-    see_this_node()
     for ni in etcd.get_all('networkinterface', None):
         if ni['state'] == 'deleted':
             continue
@@ -366,7 +373,6 @@ def get_instance_interfaces(instance_uuid):
 
 
 def get_network_interfaces(network_uuid):
-    see_this_node()
     for ni in etcd.get_all('networkinterface', None):
         if ni['state'] == 'deleted':
             continue
@@ -375,12 +381,10 @@ def get_network_interfaces(network_uuid):
 
 
 def get_interface(interface_uuid):
-    see_this_node()
     return etcd.get('networkinterface', None, interface_uuid)
 
 
 def update_network_interface_state(interface_uuid, state):
-    see_this_node
     ni = get_interface(interface_uuid)
     ni['state'] = state
     ni['state_updated'] = time.time()
@@ -391,21 +395,18 @@ def update_network_interface_state(interface_uuid, state):
 
 
 def add_floating_to_interface(interface_uuid, addr):
-    see_this_node
     ni = get_interface(interface_uuid)
     ni['floating'] = addr
     etcd.put('networkinterface', None, interface_uuid, ni)
 
 
 def remove_floating_from_interface(interface_uuid):
-    see_this_node
     ni = get_interface(interface_uuid)
     ni['floating'] = None
     etcd.put('networkinterface', None, interface_uuid, ni)
 
 
 def create_snapshot(snapshot_uuid, device, instance_uuid, created):
-    see_this_node()
     etcd.put(
         'snapshot', instance_uuid, created,
         {
@@ -417,15 +418,21 @@ def create_snapshot(snapshot_uuid, device, instance_uuid, created):
 
 
 def get_instance_snapshots(instance_uuid):
-    see_this_node()
     for m in etcd.get_all('snapshot', instance_uuid,
                           sort_order='ascend'):
         yield m
 
 
 def add_event(object_type, object_uuid, operation, phase, duration, message):
-    see_this_node()
     t = time.time()
+    LOG.withLabel(object_type, object_uuid).withFields(
+        {
+            'fqdn': config.parsed.get('NODE_NAME'),
+            'operation': operation,
+            'phase': phase,
+            'duration': duration,
+            'message': message
+        }).info('Added event')
     etcd.put(
         'event/%s' % object_type, object_uuid, t,
         {
@@ -441,14 +448,12 @@ def add_event(object_type, object_uuid, operation, phase, duration, message):
 
 
 def get_events(object_type, object_uuid):
-    see_this_node()
     for m in etcd.get_all('event/%s' % object_type, object_uuid,
                           sort_order='ascend'):
         yield m
 
 
 def update_metrics_bulk(metrics):
-    see_this_node()
     node = config.parsed.get('NODE_NAME')
     etcd.put(
         'metrics', node, None,
@@ -461,21 +466,32 @@ def update_metrics_bulk(metrics):
 
 
 def get_metrics(fqdn):
-    see_this_node()
     d = etcd.get('metrics', fqdn, None)
-    return d['metrics']
+    if not d:
+        return {}
+    return d.get('metrics', {})
+
+
+def _port_free(port):
+    s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
+    try:
+        s.bind(('0.0.0.0', port))  # lgtm
+        return True
+    except socket.error:
+        return False
+    finally:
+        s.close()
 
 
 def allocate_console_port(instance_uuid):
-    see_this_node()
     node = config.parsed.get('NODE_NAME')
-    with etcd.get_lock('sf/console/%s' % node) as _:
+    with etcd.get_lock('console', None, node):
         consumed = []
         for value in etcd.get_all('console', node):
             consumed.append(value['port'])
 
         port = random.randint(30000, 50000)
-        while port in consumed:
+        while port in consumed or not _port_free(port):
             port = random.randint(30000, 50000)
 
         etcd.put(
@@ -488,7 +504,6 @@ def allocate_console_port(instance_uuid):
 
 
 def free_console_port(port):
-    see_this_node()
     etcd.delete('console', config.parsed.get('NODE_NAME'), port)
 
 
@@ -526,3 +541,49 @@ def persist_node_vxid_mapping(node, vxid_to_mac):
 
 def get_node_vxid_mapping(node):
     etcd.get('vxid_mapping', None, node)
+
+
+def enqueue(queuename, workitem):
+    etcd.enqueue(queuename, workitem)
+
+
+def enqueue_instance_delete(instance_uuid):
+    enqueue_instance_delete_remote(config.parsed.get('NODE_NAME'),
+                                   instance_uuid)
+
+
+def enqueue_instance_delete_remote(node, instance_uuid):
+    enqueue(node, {
+        'tasks': [
+            DeleteInstanceTask(instance_uuid)
+        ],
+    })
+
+
+def enqueue_instance_error(instance_uuid, error_msg):
+    enqueue(config.parsed.get('NODE_NAME'), {
+        'tasks': [
+            ErrorInstanceTask(instance_uuid, error_msg)
+        ],
+    })
+
+
+def dequeue(queuename):
+    try:
+        return etcd.dequeue(queuename)
+    except exceptions.LockException:
+        # We didn't acquire the lock, we should just try again later. This probably
+        # indicates congestion.
+        return None, None
+
+
+def resolve(queuename, jobname):
+    etcd.resolve(queuename, jobname)
+
+
+def get_queue_length(queuename):
+    return etcd.get_queue_length(queuename)
+
+
+def restart_queues():
+    etcd.restart_queues()
