@@ -2,7 +2,6 @@ import mock
 import testtools
 
 
-from shakenfist import config
 from shakenfist.daemons import cleaner
 
 
@@ -61,7 +60,9 @@ def fake_exists(path):
 def fake_config(key):
     fc = {
         'NODE_NAME': 'abigcomputer',
-        'STORAGE_PATH': '/srv/shakenfist'
+        'STORAGE_PATH': '/srv/shakenfist',
+        'LOGLEVEL_CLEANER': 'debug',
+        'LOG_METHOD_TRACE': 1,
     }
 
     if key in fc:
@@ -111,7 +112,7 @@ class CleanerTestCase(testtools.TestCase):
     @mock.patch('time.time', return_value=7)
     def test_update_power_states(self, mock_time, mock_exists, mock_put,
                                  mock_get, mock_event, mock_see):
-        m = cleaner.monitor()
+        m = cleaner.Monitor('cleaner')
         m._update_power_states()
 
         self.assertEqual(
@@ -123,7 +124,8 @@ class CleanerTestCase(testtools.TestCase):
                               'power_state_previous': 'unknown',
                               'power_state': 'on',
                               'power_state_updated': 7,
-                              'video': {'memory': 16384, 'model': 'cirrus'}
+                              'video': {'memory': 16384, 'model': 'cirrus'},
+                              'error_message': None,
                           }),
                 mock.call('instance', None, 'shutoff',
                           {
@@ -132,7 +134,8 @@ class CleanerTestCase(testtools.TestCase):
                               'power_state_previous': 'unknown',
                               'power_state': 'off',
                               'power_state_updated': 7,
-                              'video': {'memory': 16384, 'model': 'cirrus'}
+                              'video': {'memory': 16384, 'model': 'cirrus'},
+                              'error_message': None,
                           }),
                 mock.call('instance', None, 'crashed',
                           {
@@ -143,7 +146,8 @@ class CleanerTestCase(testtools.TestCase):
                               'power_state_updated': 7,
                               'state': 'error',
                               'state_updated': 7,
-                              'video': {'memory': 16384, 'model': 'cirrus'}
+                              'video': {'memory': 16384, 'model': 'cirrus'},
+                              'error_message': None,
                           }),
                 mock.call('instance', None, 'crashed',
                           {
@@ -154,7 +158,8 @@ class CleanerTestCase(testtools.TestCase):
                               'power_state_updated': 7,
                               'state': 'error',
                               'state_updated': 7,
-                              'video': {'memory': 16384, 'model': 'cirrus'}
+                              'video': {'memory': 16384, 'model': 'cirrus'},
+                              'error_message': None,
                           }),
                 mock.call('instance', None, 'paused',
                           {
@@ -163,7 +168,8 @@ class CleanerTestCase(testtools.TestCase):
                               'power_state_previous': 'unknown',
                               'power_state': 'paused',
                               'power_state_updated': 7,
-                              'video': {'memory': 16384, 'model': 'cirrus'}
+                              'video': {'memory': 16384, 'model': 'cirrus'},
+                              'error_message': None,
                           }),
                 mock.call('instance', None, 'suspended',
                           {
@@ -172,7 +178,8 @@ class CleanerTestCase(testtools.TestCase):
                               'power_state_previous': 'unknown',
                               'power_state': 'paused',
                               'power_state_updated': 7,
-                              'video': {'memory': 16384, 'model': 'cirrus'}
+                              'video': {'memory': 16384, 'model': 'cirrus'},
+                              'error_message': None,
                           }),
                 mock.call('instance', None, 'foo',
                           {
@@ -181,7 +188,8 @@ class CleanerTestCase(testtools.TestCase):
                               'power_state_previous': 'unknown',
                               'power_state': 'off',
                               'power_state_updated': 7,
-                              'video': {'memory': 16384, 'model': 'cirrus'}
+                              'video': {'memory': 16384, 'model': 'cirrus'},
+                              'error_message': None,
                           }),
                 mock.call('instance', None, 'bar',
                           {
@@ -190,7 +198,8 @@ class CleanerTestCase(testtools.TestCase):
                               'power_state_previous': 'unknown',
                               'power_state': 'off',
                               'power_state_updated': 7,
-                              'video': {'memory': 16384, 'model': 'cirrus'}
+                              'video': {'memory': 16384, 'model': 'cirrus'},
+                              'error_message': None,
                           }),
                 mock.call('instance', None, 'nofiles',
                           {
@@ -198,7 +207,8 @@ class CleanerTestCase(testtools.TestCase):
                               'node': 'abigcomputer',
                               'state': 'error',
                               'state_updated': 7,
-                              'video': {'memory': 16384, 'model': 'cirrus'}
+                              'video': {'memory': 16384, 'model': 'cirrus'},
+                              'error_message': None,
                           })
             ],
             mock_put.mock_calls)
