@@ -3,7 +3,6 @@ import testtools
 import time
 
 
-from shakenfist import config
 from shakenfist import db
 
 
@@ -11,14 +10,12 @@ class DBTestCase(testtools.TestCase):
     def setUp(self):
         super(DBTestCase, self).setUp()
 
-    @mock.patch('shakenfist.db.see_this_node')
     @mock.patch('shakenfist.etcd.put')
     @mock.patch('shakenfist.db.allocate_console_port', side_effect=[1, 2])
-    def test_create_instance(self, mock_console_allocate, mock_put, mock_see):
+    def test_create_instance(self, mock_console_allocate, mock_put):
         db.create_instance('uuid42', 'barry', 1, 2048, 'disks',
                            'sshkey', 'userdata', 'namespace',
-                           {'memory': 16384, 'model': 'cirrus'})
-        mock_see.assert_called()
+                           {'memory': 16384, 'model': 'cirrus'}, None)
 
         etcd_write = mock_put.mock_calls[0][1]
         del etcd_write[3]['node']
@@ -40,17 +37,19 @@ class DBTestCase(testtools.TestCase):
                  'state': 'initial',
                  'namespace': 'namespace',
                  'power_state': 'initial',
-                 'video': {'memory': 16384, 'model': 'cirrus'}
+                 'video': {'memory': 16384, 'model': 'cirrus'},
+                 'node_history': [],
+                 'error_message': None,
+                 'requested_placement': None,
+                 'placement_attempts': 0,
              }),
             etcd_write)
 
-    @mock.patch('shakenfist.db.see_this_node')
     @mock.patch('shakenfist.etcd.get',
                 return_value={'uuid': 'uuid42', 'state': 'initial'})
     @mock.patch('shakenfist.etcd.put')
-    def test_update_instance_state(self, mock_put, mock_get, mock_see):
+    def test_update_instance_state(self, mock_put, mock_get):
         db.update_instance_state('uuid42', 'created')
-        mock_see.assert_called()
         mock_get.assert_called()
 
         etcd_write = mock_put.mock_calls[0][1]
@@ -61,27 +60,24 @@ class DBTestCase(testtools.TestCase):
             {
                 'state': 'created',
                 'uuid': 'uuid42',
-                'video': {'memory': 16384, 'model': 'cirrus'}
+                'video': {'memory': 16384, 'model': 'cirrus'},
+                'error_message': None,
             },
             etcd_write[3])
 
-    @mock.patch('shakenfist.db.see_this_node')
     @mock.patch('shakenfist.etcd.get',
                 return_value={'uuid': 'uuid42', 'state': 'created'})
     @mock.patch('shakenfist.etcd.put')
-    def test_update_instance_state_duplicate(self, mock_put, mock_get, mock_see):
+    def test_update_instance_state_duplicate(self, mock_put, mock_get):
         db.update_instance_state('uuid42', 'created')
-        mock_see.assert_called()
         mock_get.assert_called()
         mock_put.assert_not_called()
 
-    @mock.patch('shakenfist.db.see_this_node')
     @mock.patch('shakenfist.etcd.get',
                 return_value={'uuid': 'uuid42', 'power_state': 'on'})
     @mock.patch('shakenfist.etcd.put')
-    def test_update_instance_power_state(self, mock_put, mock_get, mock_see):
+    def test_update_instance_power_state(self, mock_put, mock_get):
         db.update_instance_power_state('uuid42', 'off')
-        mock_see.assert_called()
         mock_get.assert_called()
 
         etcd_write = mock_put.mock_calls[0][1]
@@ -93,21 +89,19 @@ class DBTestCase(testtools.TestCase):
                 'power_state': 'off',
                 'power_state_previous': 'on',
                 'uuid': 'uuid42',
-                'video': {'memory': 16384, 'model': 'cirrus'}
+                'video': {'memory': 16384, 'model': 'cirrus'},
+                'error_message': None,
             },
             etcd_write[3])
 
-    @mock.patch('shakenfist.db.see_this_node')
     @mock.patch('shakenfist.etcd.get',
                 return_value={'uuid': 'uuid42', 'power_state': 'on'})
     @mock.patch('shakenfist.etcd.put')
-    def test_update_instance_power_state_duplicate(self, mock_put, mock_get, mock_see):
+    def test_update_instance_power_state_duplicate(self, mock_put, mock_get):
         db.update_instance_power_state('uuid42', 'on')
-        mock_see.assert_called()
         mock_get.assert_called()
         mock_put.assert_not_called()
 
-    @mock.patch('shakenfist.db.see_this_node')
     @mock.patch('shakenfist.etcd.get',
                 return_value={
                     'uuid': 'uuid42',
@@ -116,13 +110,11 @@ class DBTestCase(testtools.TestCase):
                     'power_state_updated': time.time()
                 })
     @mock.patch('shakenfist.etcd.put')
-    def test_update_instance_power_state_transition_new(self, mock_put, mock_get, mock_see):
+    def test_update_instance_power_state_transition_new(self, mock_put, mock_get):
         db.update_instance_power_state('uuid42', 'on')
-        mock_see.assert_called()
         mock_get.assert_called()
         mock_put.assert_not_called()
 
-    @mock.patch('shakenfist.db.see_this_node')
     @mock.patch('shakenfist.etcd.get',
                 return_value={
                     'uuid': 'uuid42',
@@ -131,9 +123,8 @@ class DBTestCase(testtools.TestCase):
                     'power_state_updated': time.time() - 71
                 })
     @mock.patch('shakenfist.etcd.put')
-    def test_update_instance_power_state_transition_old(self, mock_put, mock_get, mock_see):
+    def test_update_instance_power_state_transition_old(self, mock_put, mock_get):
         db.update_instance_power_state('uuid42', 'on')
-        mock_see.assert_called()
         mock_get.assert_called()
 
         etcd_write = mock_put.mock_calls[0][1]
@@ -145,6 +136,7 @@ class DBTestCase(testtools.TestCase):
                 'power_state': 'on',
                 'power_state_previous': 'transition-to-off',
                 'uuid': 'uuid42',
-                'video': {'memory': 16384, 'model': 'cirrus'}
+                'video': {'memory': 16384, 'model': 'cirrus'},
+                'error_message': None,
             },
             etcd_write[3])
