@@ -25,6 +25,14 @@ class FakeScheduler(object):
         return config.parsed.get('NODE_NAME')
 
 
+class FakeInstance(object):
+    def __init__(self, namespace=None):
+        self.db_entry = {'namespace': namespace}
+
+    def unique_label(self):
+        return ('instance', self.db_entry['uuid'])
+
+
 def _encode_key(key):
     return bcrypt.hashpw(key.encode('utf-8'), bcrypt.gensalt())
 
@@ -344,25 +352,6 @@ class ExternalApiGeneralTestCase(ExternalApiTestCase):
             },
             resp.get_json())
 
-    @mock.patch('shakenfist.db.get_instances',
-                return_value=[{'uuid': '123', 'state': 'deleted'}])
-    @mock.patch('shakenfist.db.get_networks',
-                return_value=[{'uuid': '123', 'state': 'deleted'}])
-    @mock.patch('shakenfist.db.hard_delete_instance')
-    @mock.patch('shakenfist.db.hard_delete_network')
-    @mock.patch('shakenfist.etcd.delete')
-    @mock.patch('shakenfist.db.get_lock')
-    def test_delete_namespace_with_deleted(self, mock_lock, mock_etcd_delete,
-                                           mock_hd_network, mock_hd_instance,
-                                           mock_get_networks, mock_get_instances):
-        resp = self.client.delete('/auth/namespaces/foo',
-                                  headers={'Authorization': self.auth_header})
-        self.assertEqual(None, resp.get_json())
-        self.assertEqual(200, resp.status_code)
-        mock_hd_instance.assert_called()
-        mock_hd_network.assert_called()
-        mock_etcd_delete.assert_called()
-
     def test_delete_namespace_key_missing_args(self):
         resp = self.client.delete('/auth/namespaces/system/',
                                   headers={'Authorization': self.auth_header})
@@ -538,7 +527,7 @@ class ExternalApiGeneralTestCase(ExternalApiTestCase):
                               'name': 'banana',
                               'namespace': 'foo'})
     @mock.patch('shakenfist.db.get_metadata', return_value={'a': 'a', 'b': 'b'})
-    def test_get_network_metadata(self, mock_get_network, mock_md_get):
+    def test_get_network_metadata(self, mock_md_get, mock_get_network):
         resp = self.client.get(
             '/networks/foo/metadata', headers={'Authorization': self.auth_header})
         self.assertEqual({'a': 'a', 'b': 'b'}, resp.get_json())
@@ -647,33 +636,33 @@ class ExternalApiGeneralTestCase(ExternalApiTestCase):
 
     @mock.patch('shakenfist.db.get_nodes',
                 return_value=[{
-                                'fqdn': 'sf-1',
-                                'ip': '192.168.72.240',
-                                'lastseen': 1594952905.2100437,
-                                'version': '0.0.1'
-                              },
-                              {
-                                'fqdn': 'sf-2',
-                                'ip': '192.168.72.230',
-                                'lastseen': 1594952904.8870885,
-                                'version': '0.0.1'
-                               }])
+                    'fqdn': 'sf-1',
+                    'ip': '192.168.72.240',
+                    'lastseen': 1594952905.2100437,
+                    'version': '0.0.1'
+                },
+                    {
+                    'fqdn': 'sf-2',
+                    'ip': '192.168.72.230',
+                    'lastseen': 1594952904.8870885,
+                    'version': '0.0.1'
+                }])
     def test_get_node(self, mock_md_get):
         resp = self.client.get('/nodes',
                                headers={'Authorization': self.auth_header})
         self.assertEqual([{
-                            'name': 'sf-1',
-                            'ip': '192.168.72.240',
-                            'lastseen': 1594952905.2100437,
-                            'version': '0.0.1'
-                          },
-                          {
-                            'name': 'sf-2',
-                            'ip': '192.168.72.230',
-                            'lastseen': 1594952904.8870885,
-                            'version': '0.0.1'
-                          }],
-                         resp.get_json())
+            'name': 'sf-1',
+            'ip': '192.168.72.240',
+            'lastseen': 1594952905.2100437,
+            'version': '0.0.1'
+        },
+            {
+            'name': 'sf-2',
+            'ip': '192.168.72.230',
+            'lastseen': 1594952904.8870885,
+            'version': '0.0.1'
+        }],
+            resp.get_json())
         self.assertEqual(200, resp.status_code)
         self.assertEqual('application/json', resp.content_type)
 
@@ -690,77 +679,271 @@ class ExternalApiInstanceTestCase(ExternalApiTestCase):
         self.mock_virt_from_db = self.virt_from_db.start()
         self.addCleanup(self.virt_from_db.stop)
 
-    @mock.patch('shakenfist.external_api.app._delete_instance')
+        def fake_config_instance(key):
+            fc = {
+                'API_ASYNC_WAIT': 1,
+                'LOG_METHOD_TRACE': 1,
+            }
+            if key in fc:
+                return fc[key]
+            raise Exception('fake_config_instance() Unknown config key')
+
+        self.config = mock.patch('shakenfist.config.parsed.get',
+                                 fake_config_instance)
+        self.mock_config = self.config.start()
+
+        self.addCleanup(self.config.stop)
+
+    @mock.patch('shakenfist.db.enqueue')
     @mock.patch('shakenfist.db.get_instances',
                 return_value=[{
-                                'namespace': 'system',
-                                'node': 'sf-2',
-                                'power_state': 'initial',
-                                'state': 'created',
-                                'uuid': '6a973b82-31b3-4780-93e4-04d99ae49f3f',
-                              },
-                              {
-                                'name': 'timma',
-                                'namespace': 'system',
-                                'node': 'sf-2',
-                                'power_state': 'initial',
-                                'state': 'created',
-                                'uuid': '847b0327-9b17-4148-b4ed-be72b6722c17',
-                              }])
+                    'namespace': 'system',
+                    'node': 'sf-2',
+                    'power_state': 'initial',
+                    'state': 'created',
+                    'uuid': '6a973b82-31b3-4780-93e4-04d99ae49f3f',
+                },
+                    {
+                    'name': 'timma',
+                    'namespace': 'system',
+                    'node': 'sf-2',
+                    'power_state': 'initial',
+                    'state': 'created',
+                    'uuid': '847b0327-9b17-4148-b4ed-be72b6722c17',
+                }])
+    @mock.patch('shakenfist.db.get_instance',
+                return_value={
+                    'state': 'deleted',
+                },)
     @mock.patch('shakenfist.etcd.put')
     @mock.patch('shakenfist.db.get_lock')
     def test_delete_all_instances(self,
                                   mock_db_get_lock,
                                   mock_etcd_put,
+                                  mock_get_instance,
                                   mock_get_instances,
-                                  mock_del_instance):
+                                  mock_enqueue):
 
         resp = self.client.delete('/instances',
                                   headers={'Authorization': self.auth_header},
                                   data=json.dumps({
-                                    'confirm': True,
-                                    'namespace': 'foo'
+                                      'confirm': True,
+                                      'namespace': 'foo'
                                   }))
         self.assertEqual(['6a973b82-31b3-4780-93e4-04d99ae49f3f',
                           '847b0327-9b17-4148-b4ed-be72b6722c17'],
                          resp.get_json())
         self.assertEqual(200, resp.status_code)
 
-    @mock.patch('shakenfist.external_api.app._delete_instance')
+    @mock.patch('shakenfist.db.enqueue')
     @mock.patch('shakenfist.db.get_instances',
                 return_value=[{
-                                'namespace': 'system',
-                                'node': 'sf-2',
-                                'power_state': 'initial',
-                                'state': 'deleted',
-                                'uuid': '6a973b82-31b3-4780-93e4-04d99ae49f3f',
-                              },
-                              {
-                                'name': 'timma',
-                                'namespace': 'system',
-                                'node': 'sf-2',
-                                'power_state': 'initial',
-                                'state': 'created',
-                                'uuid': '847b0327-9b17-4148-b4ed-be72b6722c17',
-                              }])
+                    'namespace': 'system',
+                    'node': 'sf-2',
+                    'power_state': 'initial',
+                    'state': 'deleted',
+                    'uuid': '6a973b82-31b3-4780-93e4-04d99ae49f3f',
+                },
+                    {
+                    'name': 'timma',
+                    'namespace': 'system',
+                    'node': 'sf-2',
+                    'power_state': 'initial',
+                    'state': 'created',
+                    'uuid': '847b0327-9b17-4148-b4ed-be72b6722c17',
+                }])
+    @mock.patch('shakenfist.db.get_instance',
+                return_value={
+                    'state': 'deleted',
+                },)
     @mock.patch('shakenfist.etcd.put')
     @mock.patch('shakenfist.db.get_lock')
     def test_delete_all_instances_one_already_deleted(self,
                                                       mock_db_get_lock,
                                                       mock_etcd_put,
+                                                      mock_get_instance,
                                                       mock_get_instances,
-                                                      mock_del_instance):
+                                                      mock_enqueue):
 
         resp = self.client.delete('/instances',
                                   headers={'Authorization': self.auth_header},
                                   data=json.dumps({
-                                    'confirm': True,
-                                    'namespace': 'foo'
+                                      'confirm': True,
+                                      'namespace': 'foo'
                                   }))
         self.assertEqual(['847b0327-9b17-4148-b4ed-be72b6722c17'],
                          resp.get_json())
         self.assertEqual(200, resp.status_code)
 
+    def test_post_instance_no_disk(self):
+        resp = self.client.post('/instances',
+                                headers={'Authorization': self.auth_header},
+                                data=json.dumps({
+                                    'name': 'test_instance',
+                                    'cpus': 1,
+                                    'memory': 1024,
+                                    'network': [],
+                                    'disk': None,
+                                    'ssh_key': None,
+                                    'user_data': None,
+                                    'placed_on': None,
+                                    'namespace': None,
+                                    'instance_uuid': None
+                                }))
+        self.assertEqual(
+            {'error': 'instance must specify at least one disk', 'status': 400},
+            resp.get_json())
+        self.assertEqual(400, resp.status_code)
+
+    def test_post_instance_invalid_disk(self):
+        resp = self.client.post('/instances',
+                                headers={'Authorization': self.auth_header},
+                                data=json.dumps({
+                                    'name': 'test_instance',
+                                    'cpus': 1,
+                                    'memory': 1024,
+                                    'network': [],
+                                    'disk': ['8@cirros'],
+                                    'ssh_key': None,
+                                    'user_data': None,
+                                    'placed_on': None,
+                                    'namespace': None,
+                                    'instance_uuid': None
+                                }))
+        self.assertEqual(
+            {'error': 'disk specification should contain JSON objects', 'status': 400},
+            resp.get_json())
+        self.assertEqual(400, resp.status_code)
+
+    def test_post_instance_invalid_network(self):
+        resp = self.client.post('/instances',
+                                headers={'Authorization': self.auth_header},
+                                data=json.dumps({
+                                    'name': 'test_instance',
+                                    'cpus': 1,
+                                    'memory': 1024,
+                                    'network': ['87c15186-5f73-4947-a9fb-2183c4951efc'],
+                                    'disk': [{'size': 8,
+                                              'base': 'cirros'}],
+                                    'ssh_key': None,
+                                    'user_data': None,
+                                    'placed_on': None,
+                                    'namespace': None,
+                                    'instance_uuid': None
+                                }))
+        self.assertEqual(
+            {'error': 'network specification should contain JSON objects', 'status': 400},
+            resp.get_json())
+        self.assertEqual(400, resp.status_code)
+
+    def test_post_instance_invalid_network_uuid(self):
+        resp = self.client.post('/instances',
+                                headers={'Authorization': self.auth_header},
+                                data=json.dumps({
+                                    'name': 'test_instance',
+                                    'cpus': 1,
+                                    'memory': 1024,
+                                    'network': [
+                                        {'uuid': '87c15186-5f73-4947-a9fb-2183c4951efc'}],
+                                    'disk': [{'size': 8,
+                                              'base': 'cirros'}],
+                                    'ssh_key': None,
+                                    'user_data': None,
+                                    'placed_on': None,
+                                    'namespace': None,
+                                    'instance_uuid': None
+                                }))
+        self.assertEqual(
+            {'error': 'network specification is missing network_uuid', 'status': 400},
+            resp.get_json())
+        self.assertEqual(400, resp.status_code)
+
+    def test_post_instance_only_system_allocates_uuids(self):
+        resp = self.client.post(
+            '/auth', data=json.dumps({'namespace': 'banana', 'key': 'foo'}))
+        self.assertEqual(200, resp.status_code)
+        non_system_auth_header = 'Bearer %s' % resp.get_json()['access_token']
+
+        resp = self.client.post('/instances',
+                                headers={
+                                    'Authorization': non_system_auth_header},
+                                data=json.dumps({
+                                    'name': 'test_instance',
+                                    'cpus': 1,
+                                    'memory': 1024,
+                                    'network': [
+                                        {'network_uuid': '87c15186-5f73-4947-a9fb-2183c4951efc'}],
+                                    'disk': [{'size': 8,
+                                              'base': 'cirros'}],
+                                    'ssh_key': None,
+                                    'user_data': None,
+                                    'placed_on': None,
+                                    'namespace': None,
+                                    'instance_uuid': 'cbc58e78-d9ec-4cd5-b417-f715849126e1'
+                                }))
+        self.assertEqual(
+            {'error': 'only system can specify an instance uuid', 'status': 401},
+            resp.get_json())
+        self.assertEqual(401, resp.status_code)
+
+    def test_post_instance_only_system_specifies_namespaces(self):
+        resp = self.client.post(
+            '/auth', data=json.dumps({'namespace': 'banana', 'key': 'foo'}))
+        self.assertEqual(200, resp.status_code)
+        non_system_auth_header = 'Bearer %s' % resp.get_json()['access_token']
+
+        resp = self.client.post('/instances',
+                                headers={
+                                    'Authorization': non_system_auth_header},
+                                data=json.dumps({
+                                    'name': 'test_instance',
+                                    'cpus': 1,
+                                    'memory': 1024,
+                                    'network': [
+                                        {'network_uuid': '87c15186-5f73-4947-a9fb-2183c4951efc'}],
+                                    'disk': [{'size': 8,
+                                              'base': 'cirros'}],
+                                    'ssh_key': None,
+                                    'user_data': None,
+                                    'placed_on': None,
+                                    'namespace': 'gerkin',
+                                    'instance_uuid': None
+                                }))
+        self.assertEqual(
+            {'error': 'only admins can create resources in a different namespace',
+             'status': 401},
+            resp.get_json())
+        self.assertEqual(401, resp.status_code)
+
+    @mock.patch('shakenfist.virt.from_db', return_value=FakeInstance(namespace='foo'))
+    @mock.patch('shakenfist.db.add_event')
+    def test_post_instance_fails_ownership(self, mock_event, mock_virt_from_db):
+        resp = self.client.post(
+            '/auth', data=json.dumps({'namespace': 'banana', 'key': 'foo'}))
+        self.assertEqual(200, resp.status_code)
+        non_system_auth_header = 'Bearer %s' % resp.get_json()['access_token']
+
+        resp = self.client.post('/instances',
+                                headers={
+                                    'Authorization': non_system_auth_header},
+                                data=json.dumps({
+                                    'name': 'test_instance',
+                                    'cpus': 1,
+                                    'memory': 1024,
+                                    'network': [
+                                        {'network_uuid': '87c15186-5f73-4947-a9fb-2183c4951efc'}],
+                                    'disk': [{'size': 8,
+                                              'base': 'cirros'}],
+                                    'ssh_key': None,
+                                    'user_data': None,
+                                    'placed_on': None,
+                                    'namespace': None,
+                                    'instance_uuid': None
+                                }))
+        self.assertEqual({'error': 'instance not found',
+                          'status': 404}, resp.get_json())
+        self.assertEqual(404, resp.status_code)
+
 
 class ExternalApiNetworkTestCase(ExternalApiTestCase):
     def setUp(self):
@@ -771,6 +954,8 @@ class ExternalApiNetworkTestCase(ExternalApiTestCase):
                 'NODE_NAME': 'seriously',
                 'NODE_IP': '127.0.0.1',
                 'NETWORK_NODE_IP': '127.0.0.1',
+                'LOG_METHOD_TRACE': 1,
+                'NODE_EGRESS_NIC': 'eth0'
             }
             if key in fc:
                 return fc[key]
@@ -785,11 +970,11 @@ class ExternalApiNetworkTestCase(ExternalApiTestCase):
 
     @mock.patch('shakenfist.db.get_networks',
                 return_value=[{
-                                'floating_gateway': '10.10.0.150',
-                                'name': 'bob',
-                                'state': 'created',
-                                'uuid': '30f6da44-look-i-am-uuid',
-                                }])
+                    'floating_gateway': '10.10.0.150',
+                    'name': 'bob',
+                    'state': 'created',
+                    'uuid': '30f6da44-look-i-am-uuid',
+                }])
     @mock.patch('shakenfist.db.get_network_interfaces', return_value=[])
     @mock.patch('shakenfist.db.get_ipmanager',
                 return_value=ipmanager.NetBlock('10.0.0.0/24'))
@@ -809,15 +994,15 @@ class ExternalApiNetworkTestCase(ExternalApiTestCase):
                                  mock_db_get_networks):
 
         mock_network = mock.patch('shakenfist.net.from_db',
-                                  return_value=net.Network())
+                                  return_value=net.Network({'uuid': 'foo'}))
         mock_network.start()
         self.addCleanup(mock_network.stop)
 
         resp = self.client.delete('/networks',
                                   headers={'Authorization': self.auth_header},
                                   data=json.dumps({
-                                    'confirm': True,
-                                    'namespace': 'foo'
+                                      'confirm': True,
+                                      'namespace': 'foo'
                                   }))
         self.assertEqual(['30f6da44-look-i-am-uuid'],
                          resp.get_json())
@@ -827,11 +1012,11 @@ class ExternalApiNetworkTestCase(ExternalApiTestCase):
 
     @mock.patch('shakenfist.db.get_networks',
                 return_value=[{
-                                'floating_gateway': '10.10.0.150',
-                                'name': 'bob',
-                                'state': 'deleted',
-                                'uuid': '30f6da44-look-i-am-uuid',
-                                }])
+                    'floating_gateway': '10.10.0.150',
+                    'name': 'bob',
+                    'state': 'deleted',
+                    'uuid': '30f6da44-look-i-am-uuid',
+                }])
     @mock.patch('shakenfist.db.get_network_interfaces', return_value=[])
     @mock.patch('shakenfist.db.get_ipmanager',
                 return_value=ipmanager.NetBlock('10.0.0.0/24'))
@@ -849,9 +1034,7 @@ class ExternalApiNetworkTestCase(ExternalApiTestCase):
         resp = self.client.delete('/networks',
                                   headers={'Authorization': self.auth_header},
                                   data=json.dumps({
-                                    'confirm': True,
-                                    'namespace': 'foo'
+                                      'confirm': True,
+                                      'namespace': 'foo'
                                   }))
-        self.assertEqual([],
-                         resp.get_json())
-        self.assertEqual(200, resp.status_code)
+        self.assertEqual([], resp.get_json())
