@@ -19,6 +19,15 @@ with open('%s/files/qemu-img-info' % TEST_DIR) as f:
 with open('%s/files/cirros-download' % TEST_DIR) as f:
     CIRROS_DOWNLOAD_HTML = f.read()
 
+with open('%s/files/cirros-MD5SUMS-0.3.4' % TEST_DIR) as f:
+    CIRROS_MD5SUM_0_3_4 = f.read()
+
+with open('%s/files/ubuntu-MD5SUMS-bionic' % TEST_DIR) as f:
+    UBUNTU_MD5SUM_BIONIC = f.read()
+
+with open('%s/files/ubuntu-MD5SUMS-groovy' % TEST_DIR) as f:
+    UBUNTU_MD5SUM_GROOVY = f.read()
+
 with open('%s/files/ubuntu-download' % TEST_DIR) as f:
     UBUNTU_DOWNLOAD_HTML = f.read()
 
@@ -37,68 +46,15 @@ class FakeResponse(object):
         pass
 
 
-class ImagesTestCase(testtools.TestCase):
-    @mock.patch('shakenfist.image_resolver_cirros.resolve',
-                return_value='!!!cirros!!!')
-    @mock.patch('shakenfist.image_resolver_ubuntu.resolve',
-                return_value='!!!ubuntu!!!')
-    def test_resolve_image(self, mock_ubuntu, mock_centos):
-        self.assertEqual('win10', images.resolve('win10'))
-        self.assertEqual('http://example.com/image',
-                         images.resolve('http://example.com/image'))
-        self.assertEqual('!!!cirros!!!',
-                         images.resolve('cirros'))
-        self.assertEqual('!!!ubuntu!!!',
-                         images.resolve('ubuntu'))
+class FakeImage(object):
+    def __init__(self, url='http://somewhere'):
+        self.url = url
 
-    @mock.patch('requests.get', return_value=FakeResponse(200, CIRROS_DOWNLOAD_HTML))
-    def test_resolve_cirros(self, mock_get):
-        u = image_resolver_cirros.resolve('cirros')
-        self.assertEqual(
-            'http://download.cirros-cloud.net/0.5.1/cirros-0.5.1-x86_64-disk.img', u)
+    def unique_label(self):
+        return ('image', self.url)
 
-        u = image_resolver_cirros.resolve('cirros:0.3.4')
-        self.assertEqual(
-            'http://download.cirros-cloud.net/0.3.4/cirros-0.3.4-x86_64-disk.img', u)
-
-        self.assertRaises(exceptions.VersionSpecificationError,
-                          image_resolver_cirros.resolve, 'cirros***')
-
-    @mock.patch('requests.get', return_value=FakeResponse(404, None))
-    def test_resolve_cirros_error(self, mock_get):
-        self.assertRaises(exceptions.HTTPError,
-                          image_resolver_cirros.resolve, 'cirros')
-
-    @mock.patch('requests.get', return_value=FakeResponse(200, UBUNTU_DOWNLOAD_HTML))
-    @mock.patch('shakenfist.image_resolver_ubuntu.UBUNTU_URL',
-                'https://cloud-images.ubuntu.com')
-    def test_resolve_ubuntu(self, mock_get):
-        u = image_resolver_ubuntu.resolve('ubuntu')
-        self.assertEqual(
-            ('https://cloud-images.ubuntu.com/groovy/current/'
-             'groovy-server-cloudimg-amd64.img'),
-            u)
-
-        u = image_resolver_ubuntu.resolve('ubuntu:bionic')
-        self.assertEqual(
-            ('https://cloud-images.ubuntu.com/bionic/current/'
-             'bionic-server-cloudimg-amd64.img'),
-            u)
-
-        u = image_resolver_ubuntu.resolve('ubuntu:18.04')
-        self.assertEqual(
-            ('https://cloud-images.ubuntu.com/bionic/current/'
-             'bionic-server-cloudimg-amd64.img'),
-            u)
-
-        self.assertRaises(exceptions.VersionSpecificationError,
-                          image_resolver_ubuntu.resolve, 'ubuntu***')
-
-    @mock.patch('requests.get', return_value=FakeResponse(404, None))
-    def test_resolve_ubuntu_error(self, mock_get):
-        self.assertRaises(exceptions.HTTPError,
-                          image_resolver_ubuntu.resolve, 'ubuntu')
 
+class ImageUtilsTestCase(testtools.TestCase):
     @mock.patch('shakenfist.config.parsed.get', return_value='/a/b/c')
     @mock.patch('os.path.exists', return_value=True)
     def test_get_cache_path(self, mock_exists, mock_config):
@@ -115,17 +71,111 @@ class ImagesTestCase(testtools.TestCase):
         mock_makedirs.assert_called_with('/a/b/c/image_cache')
         self.assertEqual('/a/b/c/image_cache', p)
 
-    def test_hash_image_url(self):
-        h = images.hash_image_url('http://example.com')
+
+class ImageResolversTestCase(testtools.TestCase):
+    @mock.patch('requests.get', side_effect=[
+        FakeResponse(200, CIRROS_DOWNLOAD_HTML),
+        FakeResponse(200, ''),  # Handle no file available
+        FakeResponse(200, CIRROS_DOWNLOAD_HTML),
+        FakeResponse(200, CIRROS_MD5SUM_0_3_4),
+        FakeResponse(200, CIRROS_DOWNLOAD_HTML),
+    ])
+    def test_resolve_cirros(self, mock_get):
+        u = image_resolver_cirros.resolve('cirros')
+        self.assertEqual(
+            ('http://download.cirros-cloud.net/0.5.1/cirros-0.5.1-x86_64-disk.img',
+             None),
+            u)
+
+        u = image_resolver_cirros.resolve('cirros:0.3.4')
+        self.assertEqual(
+            ('http://download.cirros-cloud.net/0.3.4/cirros-0.3.4-x86_64-disk.img',
+             'ee1eca47dc88f4879d8a229cc70a07c6'),
+            u)
+
+        self.assertRaises(exceptions.VersionSpecificationError,
+                          image_resolver_cirros.resolve, 'cirros***')
+
+    @mock.patch('requests.get', return_value=FakeResponse(404, None))
+    def test_resolve_cirros_error(self, mock_get):
+        self.assertRaises(exceptions.HTTPError,
+                          image_resolver_cirros.resolve, 'cirros')
+
+    @mock.patch('requests.get', side_effect=[
+        FakeResponse(200, UBUNTU_DOWNLOAD_HTML),
+        FakeResponse(200, UBUNTU_MD5SUM_GROOVY),
+        FakeResponse(200, UBUNTU_DOWNLOAD_HTML),
+        FakeResponse(200, UBUNTU_MD5SUM_BIONIC),
+        FakeResponse(200, UBUNTU_DOWNLOAD_HTML),
+        FakeResponse(200, UBUNTU_MD5SUM_BIONIC),
+        FakeResponse(200, UBUNTU_DOWNLOAD_HTML),
+    ])
+    @mock.patch('shakenfist.image_resolver_ubuntu.UBUNTU_URL',
+                'https://cloud-images.ubuntu.com')
+    def test_resolve_ubuntu(self, mock_get):
+        u = image_resolver_ubuntu.resolve('ubuntu')
+        self.assertEqual(
+            ('https://cloud-images.ubuntu.com/groovy/current/'
+             'groovy-server-cloudimg-amd64.img',
+             '1c19b08060b9feb1cd0e7ee28fd463fb'),
+            u)
+
+        u = image_resolver_ubuntu.resolve('ubuntu:bionic')
+        self.assertEqual(
+            ('https://cloud-images.ubuntu.com/bionic/current/'
+             'bionic-server-cloudimg-amd64.img',
+             'ed44b9745b8d62bcbbc180b5f36c24bb'),
+            u)
+
+        u = image_resolver_ubuntu.resolve('ubuntu:18.04')
+        self.assertEqual(
+            ('https://cloud-images.ubuntu.com/bionic/current/'
+             'bionic-server-cloudimg-amd64.img',
+             'ed44b9745b8d62bcbbc180b5f36c24bb'),
+            u)
+
+        self.assertRaises(exceptions.VersionSpecificationError,
+                          image_resolver_ubuntu.resolve, 'ubuntu***')
+
+    @mock.patch('requests.get', return_value=FakeResponse(404, None))
+    def test_resolve_ubuntu_error(self, mock_get):
+        self.assertRaises(exceptions.HTTPError,
+                          image_resolver_ubuntu.resolve, 'ubuntu')
+
+
+class ImageObjectTestCase(testtools.TestCase):
+    @mock.patch('shakenfist.image_resolver_cirros.resolve',
+                return_value=('!!!cirros!!!', '123abc'))
+    @mock.patch('shakenfist.image_resolver_ubuntu.resolve',
+                return_value=('!!!ubuntu!!!', '123abc'))
+    @mock.patch('os.path.exists', return_value=False)
+    @mock.patch('os.makedirs')
+    def test_resolve_image(self, mock_mkdirs, mock_exists, mock_ubuntu, mock_centos):
+        img = images.Image('win10')
+        self.assertEqual('win10', img.url)
+
+        img = images.Image('http://example.com/image')
+        self.assertEqual('http://example.com/image', img.url)
+
+        img = images.Image('cirros')
+        self.assertEqual('!!!cirros!!!', img.url)
+
+        img = images.Image('ubuntu')
+        self.assertEqual('!!!ubuntu!!!', img.url)
+
+    @mock.patch('os.path.exists', return_value=False)
+    @mock.patch('os.makedirs')
+    def test_hash_image(self, mock_mkdirs, mock_exists):
+        img = images.Image('http://example.com')
         self.assertEqual('f0e6a6a97042a4f1f1c87f5f7d44315b2d'
-                         '852c2df5c7991cc66241bf7072d1c4', h)
+                         '852c2df5c7991cc66241bf7072d1c4', img.hashed_image_url)
 
     @mock.patch('requests.get',
                 return_value=FakeResponse(
                     200, '',
                     headers={'Last-Modified': 'Tue, 10 Sep 2019 07:24:40 GMT',
                              'Content-Length': 200000}))
-    @mock.patch('shakenfist.images._read_info',
+    @mock.patch('shakenfist.images.Image._read_local_info',
                 return_value={
                     'url': 'http://example.com',
                     'Last-Modified': 'Tue, 10 Sep 2019 07:24:40 GMT',
@@ -133,17 +183,18 @@ class ImagesTestCase(testtools.TestCase):
                     'version': 0
                 })
     @mock.patch('os.makedirs')
-    def test_does_not_require_fetch(self, mock_mkdirs, mock_read_info, mock_request_head):
-        _, info, image_dirty, _ = images.requires_fetch('http://example.com')
-        self.assertEqual(0, info['version'])
-        self.assertEqual(False, image_dirty)
+    def test_does_not_require_fetch(self, mock_mkdirs, mock_read_local_info, mock_request_head):
+        img = images.Image('http://example.com')
+        dirty_fields, _ = img._requires_fetch()
+        self.assertEqual(0, img.info['version'])
+        self.assertEqual({}, dirty_fields)
 
     @mock.patch('requests.get',
                 return_value=FakeResponse(
                     200, '',
                     headers={'Last-Modified': 'Tue, 10 Sep 2019 07:24:40 GMT',
                              'Content-Length': 200000}))
-    @mock.patch('shakenfist.images._read_info',
+    @mock.patch('shakenfist.images.Image._read_local_info',
                 return_value={
                     'url': 'http://example.com',
                     'Last-Modified': 'Tue, 10 Sep 2017 07:24:40 GMT',
@@ -151,10 +202,14 @@ class ImagesTestCase(testtools.TestCase):
                     'version': 0
                 })
     @mock.patch('os.makedirs')
-    def test_requires_fetch(self, mock_mkdirs, mock_read_info, mock_request_head):
-        _, info, image_dirty, _ = images.requires_fetch('http://example.com')
-        self.assertEqual(0, info['version'])
-        self.assertEqual(True, image_dirty)
+    def test_requires_fetch(self, mock_mkdirs, mock_read_local_info, mock_request_head):
+        img = images.Image('http://example.com')
+        dirty_fields, _ = img._requires_fetch()
+        self.assertEqual(0, img.info['version'])
+        self.assertEqual({
+            'Last-Modified': {'after': 'Tue, 10 Sep 2019 07:24:40 GMT',
+                              'before': 'Tue, 10 Sep 2017 07:24:40 GMT'}
+        }, dirty_fields)
 
     @mock.patch('shakenfist.config.parsed.get', return_value='/a/b/c')
     @mock.patch('os.path.exists', return_value=False)
@@ -166,8 +221,12 @@ class ImagesTestCase(testtools.TestCase):
                              'Content-Length': 200000}))
     def test_fetch_image_new(self, mock_get, mock_makedirs,
                              mock_exists, mock_config):
-        _, _, image_dirty, _ = images.requires_fetch('http://example.com')
-        self.assertEqual(True, image_dirty)
+        img = images.Image('http://example.com')
+        dirty_fields, _ = img._requires_fetch()
+        self.assertEqual({
+            'Content-Length': {'after': 200000, 'before': None},
+            'Last-Modified': {'after': 'Tue, 10 Sep 2019 07:24:40 GMT', 'before': None}
+        }, dirty_fields)
 
     @mock.patch('shakenfist.config.parsed.get', return_value='/a/b/c')
     @mock.patch('os.path.exists', return_value=True)
@@ -186,104 +245,108 @@ class ImagesTestCase(testtools.TestCase):
         mock_open = mock.mock_open()
         with mock.patch.object(six.moves.builtins, 'open',
                                new=mock_open):
-            _, _, image_dirty, _ = images.requires_fetch('http://example.com')
-        self.assertEqual(False, image_dirty)
+            img = images.Image('http://example.com')
+            dirty_fields, _ = img._requires_fetch()
+        self.assertEqual({}, dirty_fields)
 
     @mock.patch('shakenfist.config.parsed.get', return_value='/a/b/c')
     @mock.patch('os.path.exists', side_effect=[True, True, False])
     @mock.patch('os.makedirs')
-    @mock.patch('shakenfist.images._read_info',
+    @mock.patch('shakenfist.images.Image._read_local_info',
                 return_value={
                     'Last-Modified': 'Tue, 10 Sep 2018 07:24:40 GMT',
                     'Content-Length': 100000,
                     'version': 0})
-    @mock.patch('oslo_concurrency.processutils.execute',
+    @mock.patch('shakenfist.util.execute',
                 return_value=(None, None))
-    def test_fetch_image_changed(self, mock_execute, mock_read_info, mock_makedirs,
+    def test_fetch_image_changed(self, mock_execute, mock_read_local_info, mock_makedirs,
                                  mock_exists, mock_config):
-        _, _, image_dirty, _ = images.requires_fetch('http://example.com')
-        self.assertEqual(True, image_dirty)
+        img = images.Image('http://example.com')
+        image_dirty, _ = img._requires_fetch()
+        self.assertEqual({
+            'Content-Length': {'after': '648', 'before': 100000},
+            'Last-Modified': {'after': 'Thu, 17 Oct 2019 07:18:26 GMT',
+                              'before': 'Tue, 10 Sep 2018 07:24:40 GMT'}
+        }, image_dirty)
 
-    @mock.patch('oslo_concurrency.processutils.execute',
+    @mock.patch('shakenfist.util.execute',
                 return_value=(None, None))
     @mock.patch('os.path.exists', return_value=True)
-    def test_transcode_image_noop(self, mock_exists, mock_execute):
-        images.transcode('/a/b/c/hash')
+    @mock.patch('shakenfist.db.get_lock')
+    @mock.patch('shakenfist.db.add_event')
+    def test_transcode_image_noop(self, mock_event, mock_lock, mock_exists, mock_execute):
+        images._transcode(None, '/a/b/c/hash', FakeImage())
         mock_execute.assert_not_called()
 
-    @mock.patch('oslo_concurrency.processutils.execute',
+    @mock.patch('shakenfist.util.execute',
                 return_value=(None, None))
     @mock.patch('os.path.exists', return_value=False)
     @mock.patch('shakenfist.images.identify',
                 return_value={'file format': 'qcow2'})
     @mock.patch('os.link')
-    def test_transcode_image_link(self, mock_link, mock_identify, mock_exists,
+    @mock.patch('shakenfist.db.get_lock')
+    @mock.patch('shakenfist.db.add_event')
+    def test_transcode_image_link(self, mock_event, mock_lock, mock_link, mock_identify, mock_exists,
                                   mock_execute):
-        images.transcode('/a/b/c/hash')
+        images._transcode(None, '/a/b/c/hash', FakeImage())
         mock_link.assert_called_with('/a/b/c/hash', '/a/b/c/hash.qcow2')
         mock_execute.assert_not_called()
 
-    @mock.patch('oslo_concurrency.processutils.execute',
+    @mock.patch('shakenfist.util.execute',
                 return_value=(None, None))
     @mock.patch('os.path.exists', return_value=False)
     @mock.patch('shakenfist.images.identify',
                 return_value={'file format': 'raw'})
     @mock.patch('os.link')
-    def test_transcode_image_convert(self, mock_link, mock_identify, mock_exists,
+    @mock.patch('shakenfist.db.get_lock')
+    @mock.patch('shakenfist.db.add_event')
+    def test_transcode_image_convert(self, mock_event, mock_lock, mock_link, mock_identify, mock_exists,
                                      mock_execute):
-        images.transcode('/a/b/c/hash')
+        images._transcode(None, '/a/b/c/hash', FakeImage())
         mock_link.assert_not_called()
         mock_execute.assert_called_with(
-            'qemu-img convert -t none -O qcow2 /a/b/c/hash /a/b/c/hash.qcow2',
-            shell=True
-        )
+            None, 'qemu-img convert -t none -O qcow2 /a/b/c/hash /a/b/c/hash.qcow2')
 
-    @mock.patch('shutil.copyfile')
-    @mock.patch('oslo_concurrency.processutils.execute',
+    @mock.patch('shakenfist.util.execute',
                 return_value=(None, None))
     @mock.patch('os.path.exists', return_value=True)
     @mock.patch('shakenfist.images.identify',
                 return_value={'virtual size': 8 * 1024 * 1024 * 1024})
     @mock.patch('os.link')
     def test_resize_image_noop(self, mock_link, mock_identify, mock_exists,
-                               mock_execute, mock_copyfile):
-        images.resize('/a/b/c/hash', 8)
+                               mock_execute):
+        images.resize(None, '/a/b/c/hash', 8)
         mock_link.assert_not_called()
         mock_execute.assert_not_called()
-        mock_copyfile.assert_not_called()
 
-    @mock.patch('shutil.copyfile')
-    @mock.patch('oslo_concurrency.processutils.execute',
+    @mock.patch('shakenfist.util.execute',
                 return_value=(None, None))
     @mock.patch('os.path.exists', return_value=False)
     @mock.patch('shakenfist.images.identify',
                 return_value={'virtual size': 8 * 1024 * 1024 * 1024})
     @mock.patch('os.link')
     def test_resize_image_link(self, mock_link, mock_identify, mock_exists,
-                               mock_execute, mock_copyfile):
-        images.resize('/a/b/c/hash', 8)
+                               mock_execute):
+        images.resize(None, '/a/b/c/hash', 8)
         mock_link.assert_called_with('/a/b/c/hash', '/a/b/c/hash.qcow2.8G')
         mock_execute.assert_not_called()
-        mock_copyfile.assert_not_called()
 
-    @mock.patch('shutil.copyfile')
-    @mock.patch('oslo_concurrency.processutils.execute',
+    @mock.patch('shakenfist.util.execute',
                 return_value=(None, None))
     @mock.patch('os.path.exists', return_value=False)
     @mock.patch('shakenfist.images.identify',
                 return_value={'virtual size': 4 * 1024 * 1024 * 1024})
     @mock.patch('os.link')
     def test_resize_image_resize(self, mock_link, mock_identify, mock_exists,
-                                 mock_execute, mock_copyfile):
-        images.resize('/a/b/c/hash', 8)
+                                 mock_execute):
+        images.resize(None, '/a/b/c/hash', 8)
         mock_link.assert_not_called()
-        mock_execute.assert_called_with(
-            'qemu-img resize /a/b/c/hash.qcow2.8G 8G', shell=True)
-        mock_copyfile.assert_called_with(
-            '/a/b/c/hash.qcow2', '/a/b/c/hash.qcow2.8G'
+        mock_execute.assert_has_calls(
+            [mock.call(None, 'cp /a/b/c/hash.qcow2 /a/b/c/hash.qcow2.8G'),
+             mock.call(None, 'qemu-img resize /a/b/c/hash.qcow2.8G 8G')]
         )
 
-    @mock.patch('oslo_concurrency.processutils.execute',
+    @mock.patch('shakenfist.util.execute',
                 return_value=(QEMU_IMG_OUT, None))
     @mock.patch('os.path.exists', return_value=True)
     def test_identify_image(self, mock_exists, mock_execute):
@@ -300,18 +363,17 @@ class ImagesTestCase(testtools.TestCase):
             'virtual size': 117440512.0
         }, d)
 
-    @mock.patch('oslo_concurrency.processutils.execute',
+    @mock.patch('shakenfist.util.execute',
                 return_value=(None, None))
     @mock.patch('os.path.exists', return_value=False)
     def test_create_cow(self, mock_exists, mock_execute):
-        images.create_cow('/a/b/c/base', '/a/b/c/cow')
+        images.create_cow(None, '/a/b/c/base', '/a/b/c/cow')
         mock_execute.assert_called_with(
-            'qemu-img create -b /a/b/c/base -f qcow2 /a/b/c/cow', shell=True)
+            None, 'qemu-img create -b /a/b/c/base -f qcow2 /a/b/c/cow')
 
-    @mock.patch('oslo_concurrency.processutils.execute',
+    @mock.patch('shakenfist.util.execute',
                 return_value=(None, None))
     def test_snapshot(self, mock_execute):
-        images.snapshot('/a/b/c/base', '/a/b/c/snap')
+        images.snapshot(None, '/a/b/c/base', '/a/b/c/snap')
         mock_execute.assert_called_with(
-            'qemu-img convert --force-share -O qcow2 /a/b/c/base /a/b/c/snap',
-            shell=True)
+            None, 'qemu-img convert --force-share -O qcow2 /a/b/c/base /a/b/c/snap')
