@@ -2,8 +2,7 @@
 
 import importlib
 import json
-import logging
-from logging import handlers as logging_handlers
+import multiprocessing
 from pbr.version import VersionInfo
 import random
 import re
@@ -16,55 +15,69 @@ import traceback
 from oslo_concurrency import processutils
 
 from shakenfist import db
+from shakenfist import config
+from shakenfist import logutil
 
 
-LOG = logging.getLogger(__file__)
-LOG.setLevel(logging.INFO)
-LOG.addHandler(logging_handlers.SysLogHandler(address='/dev/log'))
+LOG, _ = logutil.setup(__name__)
 
 
 class RecordedOperation():
-    def __init__(self, operation, object):
+    def __init__(self, operation, relatedobject):
         self.operation = operation
-        self.object = object
+        self.object = relatedobject
 
     def __enter__(self):
         self.start_time = time.time()
-        LOG.debug('%s: Start %s' % (self.object, self.operation))
-
-        object_type, object_uuid = self.get_describing_tuple()
-        db.add_event(object_type, object_uuid,
-                     self.operation, 'start', None, None)
+        object_type, object_uuid = self.unique_label()
+        if object_type and object_uuid:
+            db.add_event(object_type, object_uuid,
+                         self.operation, 'start', None, None)
         return self
 
     def __exit__(self, *args):
         duration = time.time() - self.start_time
-        LOG.info('%s: Finish %s, duration %.02f seconds'
-                 % (self.object, self.operation,
-                    duration))
+        log = LOG
+        object_type, object_uuid = self.unique_label()
+        if object_uuid:
+            if object_type:
+                db.add_event(object_type, object_uuid,
+                             self.operation, 'finish', duration, None)
+                log = LOG.withObj(self.object)
+            else:
+                log = LOG.withField({'label', self.object})
+        log.withField('duration', duration).info('Finish %s', self.operation)
 
-        object_type, object_uuid = self.get_describing_tuple()
-        db.add_event(object_type, object_uuid,
-                     self.operation, 'finish', duration, None)
-
-    def get_describing_tuple(self):
+    def unique_label(self):
         if self.object:
             if isinstance(self.object, str):
-                object_type = 'null'
+                object_type = None
                 object_uuid = self.object
             else:
-                object_type, object_uuid = self.object.get_describing_tuple()
+                object_type, object_uuid = self.object.unique_label()
         else:
-            object_type = 'null'
-            object_uuid = 'null'
+            object_type = None
+            object_uuid = None
 
         return object_type, object_uuid
 
 
-def check_for_interface(name):
-    _, stderr = processutils.execute(
-        'ip link show %s' % name, check_exit_code=[0, 1], shell=True)
-    return not stderr.rstrip('\n').endswith(' does not exist.')
+def is_network_node():
+    """Test if this node is the network node"""
+    return config.parsed.get('NODE_IP') == config.parsed.get('NETWORK_NODE_IP')
+
+
+def check_for_interface(name, up=False):
+    stdout, stderr = execute(None,
+                             'ip link show %s' % name, check_exit_code=[0, 1])
+
+    if stderr.rstrip('\n').endswith(' does not exist.'):
+        return False
+
+    if up:
+        return bool(re.match(r'.*[<,]UP[,>].*', stdout))
+
+    return True
 
 
 def get_interface_addresses(namespace, name):
@@ -72,13 +85,13 @@ def get_interface_addresses(namespace, name):
     if namespace:
         in_namespace = 'ip netns exec %s ' % namespace
 
-    stdout, _ = processutils.execute(
-        '%(in_namespace)sip addr show %(name)s'
-        % {
-            'in_namespace': in_namespace,
-            'name': name
-        },
-        check_exit_code=[0, 1], shell=True)
+    stdout, _ = execute(None,
+                        '%(in_namespace)sip addr show %(name)s'
+                        % {
+                            'in_namespace': in_namespace,
+                            'name': name
+                        },
+                        check_exit_code=[0, 1])
     if not stdout:
         return
 
@@ -91,9 +104,47 @@ def get_interface_addresses(namespace, name):
     return
 
 
+def get_default_routes(namespace):
+    in_namespace = ''
+    if namespace:
+        in_namespace = 'ip netns exec %s ' % namespace
+
+    stdout, _ = execute(None,
+                        '%(in_namespace)sip route list default'
+                        % {
+                            'in_namespace': in_namespace
+                        })
+    if not stdout:
+        return []
+
+    routes = []
+    for line in stdout.split('\n'):
+        elems = line.split(' ')
+        if len(elems) > 3 and elems[2] not in routes:
+            routes.append(elems[2])
+    return routes
+
+
+def get_safe_interface_name(interface):
+    if len(interface) > 15:
+        orig_interface = interface
+        interface = interface[:15]
+        LOG.info('Interface name truncated from %s to %s'
+                 % (orig_interface, interface))
+    return interface
+
+
+def create_interface(interface, interface_type, extra):
+    interface = get_safe_interface_name(interface)
+    execute(None,
+            'ip link add %(interface)s type %(interface_type)s %(extra)s'
+            % {'interface': interface,
+               'interface_type': interface_type,
+               'extra': extra})
+
+
 def nat_rules_for_ipblock(ipblock):
-    out, _ = processutils.execute(
-        'iptables -t nat -L POSTROUTING -n -v', shell=True)
+    out, _ = execute(None, 'iptables -t nat -L POSTROUTING -n -v')
     # Output looks like this:
     # Chain POSTROUTING (policy ACCEPT 199 packets, 18189 bytes)
     # pkts bytes target     prot opt in     out     source               destination
@@ -136,7 +187,7 @@ def extract_power_state(libvirt, domain):
 
 
 def get_api_token(base_url, namespace='system'):
-    with db.get_lock('sf/namespace/%s' % namespace) as _:
+    with db.get_lock('namespace', None, namespace):
         auth_url = base_url + '/auth'
         LOG.info('Fetching %s auth token from %s' % (namespace, auth_url))
         ns = db.get_namespace(namespace)
@@ -188,8 +239,7 @@ def discover_interfaces():
     link_ether = None
     link_ether_re = re.compile('^    link/ether (.*) brd .*')
 
-    stdout, _ = processutils.execute(
-        'ip addr list', shell=True)
+    stdout, _ = execute(None, 'ip addr list')
     for line in stdout.split('\n'):
         line = line.rstrip()
 
@@ -218,3 +268,35 @@ def ignore_exception(processname, e):
         msg += '\n%s' % traceback.format_exc()
 
     LOG.error(msg)
+
+
+def _lock_refresher(locks):
+    while True:
+        db.refresh_locks(locks)
+        time.sleep(10)
+
+
+def execute(locks, command, check_exit_code=[0], env_variables=None):
+    if not locks:
+        return processutils.execute(
+            command, check_exit_code=check_exit_code,
+            env_variables=env_variables, shell=True)
+
+    else:
+        p = multiprocessing.Process(
+            target=_lock_refresher, args=(locks,))
+        p.start()
+
+        try:
+            return processutils.execute(
+                command, check_exit_code=check_exit_code,
+                env_variables=env_variables, shell=True)
+        finally:
+            p.terminate()
+            p.join()
+
+
+def random_macaddr():
+    return '02:00:00:%02x:%02x:%02x' % (random.randint(0, 255),
+                                        random.randint(0, 255),
+                                        random.randint(0, 255))
