@@ -2,8 +2,6 @@
 
 import base64
 import jinja2
-import logging
-from logging import handlers as logging_handlers
 import io
 import json
 import os
@@ -12,25 +10,23 @@ import shutil
 import time
 import uuid
 
-from oslo_concurrency import processutils
-
 from shakenfist import config
 from shakenfist import db
+from shakenfist import exceptions
 from shakenfist import images
+from shakenfist import logutil
 from shakenfist import net
 from shakenfist import util
 
 
-LOG = logging.getLogger(__file__)
-LOG.setLevel(logging.INFO)
-LOG.addHandler(logging_handlers.SysLogHandler(address='/dev/log'))
+LOG, _ = logutil.setup(__name__)
 
 
 def from_definition(uuid=None, name=None, disks=None, memory_mb=None,
                     vcpus=None, ssh_key=None, user_data=None, owner=None,
-                    video=None):
+                    video=None, requested_placement=None):
     db_entry = db.create_instance(uuid, name, vcpus, memory_mb, disks,
-                                  ssh_key, user_data, owner, video)
+                                  ssh_key, user_data, owner, video, requested_placement)
     return Instance(db_entry)
 
 
@@ -68,7 +64,6 @@ def _get_defaulted_disk_type(disk):
 class Instance(object):
     def __init__(self, db_entry):
         self.db_entry = db_entry
-        self.tenant = None
 
         self.instance_path = os.path.join(
             config.parsed.get('STORAGE_PATH'), 'instances', self.db_entry['uuid'])
@@ -82,15 +77,15 @@ class Instance(object):
     def __str__(self):
         return 'instance(%s)' % self.db_entry['uuid']
 
-    def get_describing_tuple(self):
+    def unique_label(self):
         return ('instance', self.db_entry['uuid'])
 
     def _populate_block_devices(self):
         disk_spec = self.db_entry['disk_spec']
         if not disk_spec:
             # This should not occur since the API will filter for zero disks.
-            LOG.error("_populate_block_devices(): Found disk spec empty: %s" %
-                      self.db_entry)
+            LOG.withObj(self).error(
+                'Found disk spec empty: %s' % self.db_entry)
 
             # Stop continuous crashing by falsely claiming disks are configured.
             self.db_entry['block_devices'] = {'finalized': True}
@@ -100,14 +95,18 @@ class Instance(object):
         root_device = _get_disk_device_base(bus) + 'a'
         config_device = _get_disk_device_base(bus) + 'b'
 
+        disk_type = 'qcow2'
+        if config.parsed.get('DISK_FORMAT') == 'flat':
+            disk_type = 'raw'
+
         self.db_entry['block_devices'] = {
             'devices': [
                 {
-                    'type': 'qcow2',
+                    'type': disk_type,
                     'size': self.db_entry['disk_spec'][0].get('size'),
                     'device': root_device,
                     'bus': bus,
-                    'path': os.path.join(self.instance_path, root_device + '.qcow2'),
+                    'path': os.path.join(self.instance_path, root_device),
                     'base': self.db_entry['disk_spec'][0].get('base'),
                     'present_as': _get_defaulted_disk_type(self.db_entry['disk_spec'][0]),
                     'snapshot_ignores': False
@@ -116,7 +115,7 @@ class Instance(object):
                     'type': 'raw',
                     'device': config_device,
                     'bus': bus,
-                    'path': os.path.join(self.instance_path, config_device + '.raw'),
+                    'path': os.path.join(self.instance_path, config_device),
                     'present_as': 'disk',
                     'snapshot_ignores': True
                 }
@@ -128,11 +127,11 @@ class Instance(object):
             bus = _get_defaulted_disk_bus(d)
             device = _get_disk_device_base(bus) + chr(ord('c') + i)
             self.db_entry['block_devices']['devices'].append({
-                'type': 'qcow2',
+                'type': disk_type,
                 'size': d.get('size'),
                 'device': device,
                 'bus': bus,
-                'path': os.path.join(self.instance_path, device + '.qcow2'),
+                'path': os.path.join(self.instance_path, device),
                 'base': d.get('base'),
                 'present_as': _get_defaulted_disk_type(d),
                 'snapshot_ignores': False
@@ -141,17 +140,21 @@ class Instance(object):
 
         self.db_entry['block_devices']['finalized'] = False
 
+    # NOTE(mikal): this method is now strictly the instance specific steps for
+    # creation. It is assumed that the image sits in local cache already, and
+    # has been transcoded to the right format. This has been done to facilitate
+    # moving to a queue and task based creation mechanism.
     def create(self, lock=None):
         db.update_instance_state(self.db_entry['uuid'], 'creating')
 
         # Ensure we have state on disk
         if not os.path.exists(self.instance_path):
-            LOG.debug('%s: Creating instance storage at %s' %
-                      (self, self.instance_path))
+            LOG.withObj(self).debug(
+                'Creating instance storage at %s' % self.instance_path)
             os.makedirs(self.instance_path)
 
         # Generate a config drive
-        with util.RecordedOperation('make config drive', self) as _:
+        with util.RecordedOperation('make config drive', self):
             self._make_config_drive(os.path.join(
                 self.instance_path, self.db_entry['block_devices']['devices'][1]['path']))
 
@@ -160,26 +163,16 @@ class Instance(object):
             modified_disks = []
             for disk in self.db_entry['block_devices']['devices']:
                 if disk.get('base'):
-                    hashed = images.hash_image_url(disk['base'])
-                    with db.get_lock('sf/images/%s' % hashed) as image_lock:
-                        with util.RecordedOperation('fetch image', self) as _:
-                            image_url = images.resolve(disk['base'])
-                            hashed_image_path, info, image_dirty, resp = \
-                                images.requires_fetch(image_url)
+                    img = images.Image(disk['base'])
+                    hashed_image_path = img.image_path()
 
-                            if image_dirty:
-                                hashed_image_path = images.fetch(hashed_image_path,
-                                                                 info, resp, locks=[lock, image_lock])
-                            else:
-                                hashed_image_path = '%s.v%03d' % (
-                                    hashed_image_path, info['version'])
-
-                    try:
-                        cd = pycdlib.PyCdlib()
-                        cd.open(hashed_image_path)
-                        disk['present_as'] = 'cdrom'
-                    except Exception:
-                        pass
+                    with util.RecordedOperation('detect cdrom images', self):
+                        try:
+                            cd = pycdlib.PyCdlib()
+                            cd.open(hashed_image_path)
+                            disk['present_as'] = 'cdrom'
+                        except Exception:
+                            pass
 
                     if disk.get('present_as', 'cdrom') == 'cdrom':
                         # There is no point in resizing or COW'ing a cdrom
@@ -191,41 +184,44 @@ class Instance(object):
                             os.link(hashed_image_path, disk['path'])
                         except OSError:
                             # Different filesystems
-                            if lock:
-                                lock.refresh()
-
-                            shutil.copyfile(hashed_image_path, disk['path'])
-
-                            if lock:
-                                lock.refresh()
+                            util.execute(
+                                [lock], 'cp %s %s' % (hashed_image_path, disk['path']))
 
                         # Due to limitations in some installers, cdroms are always on IDE
                         disk['device'] = 'hd%s' % disk['device'][-1]
                         disk['bus'] = 'ide'
                     else:
-                        with util.RecordedOperation('transcode image', self) as _:
-                            if lock:
-                                lock.refresh()
-
-                            images.transcode(hashed_image_path)
-
-                        with util.RecordedOperation('resize image', self) as _:
-                            if lock:
-                                lock.refresh()
-
+                        with util.RecordedOperation('resize image', self):
                             resized_image_path = images.resize(
-                                hashed_image_path, disk['size'])
+                                [lock], hashed_image_path, disk['size'])
 
-                        with util.RecordedOperation('create copy on write layer', self) as _:
-                            if lock:
-                                lock.refresh
+                        if config.parsed.get('DISK_FORMAT') == 'qcow':
+                            with util.RecordedOperation('create copy on write layer', self):
+                                images.create_cow(
+                                    [lock], resized_image_path, disk['path'])
 
-                            images.create_cow(resized_image_path, disk['path'])
+                            # Record the backing store for modern libvirts
+                            disk['backing'] = ('<backingStore type=\'file\'>\n'
+                                               '        <format type=\'qcow2\'/>\n'
+                                               '        <source file=\'%s\'/>\n'
+                                               '      </backingStore>' % resized_image_path)
+
+                        elif config.parsed.get('DISK_FORMAT') == 'qcow_flat':
+                            with util.RecordedOperation('create flat layer', self):
+                                images.create_flat(
+                                    [lock], resized_image_path, disk['path'])
+
+                        elif config.parsed.get('DISK_FORMAT') == 'flat':
+                            with util.RecordedOperation('create raw disk', self):
+                                images.create_raw(
+                                    [lock], resized_image_path, disk['path'])
+
+                        else:
+                            raise Exception('Unknown disk format')
 
                 elif not os.path.exists(disk['path']):
-                    processutils.execute('qemu-img create -f qcow2 %s %sG'
-                                         % (disk['path'], disk['size']),
-                                         shell=True)
+                    util.execute(None, 'qemu-img create -f qcow2 %s %sG'
+                                 % (disk['path'], disk['size']))
 
                 modified_disks.append(disk)
 
@@ -236,40 +232,53 @@ class Instance(object):
             self.db_entry['uuid'], self.db_entry['block_devices'])
 
         # Create the actual instance
-        with util.RecordedOperation('create domain XML', self) as _:
+        with util.RecordedOperation('create domain XML', self):
             self._create_domain_xml()
-        with util.RecordedOperation('create domain', self) as _:
-            self.power_on()
 
+        # Sometimes on Ubuntu 20.04 we need to wait for port binding to work.
+        # Revisiting this is tracked by issue 320 on github.
+        with util.RecordedOperation('create domain', self):
+            if not self.power_on():
+                attempts = 0
+                while not self.power_on() and attempts < 100:
+                    LOG.withObj(self).warning(
+                        'Instance required an additional attempt to power on')
+                    time.sleep(5)
+                    attempts += 1
+
+        if self.is_powered_on():
+            LOG.withObj(self).info('Instance now powered on')
+        else:
+            LOG.withObj(self).info('Instance failed to power on')
         db.update_instance_state(self.db_entry['uuid'], 'created')
 
     def delete(self):
-        with util.RecordedOperation('delete domain', self) as _:
+        with util.RecordedOperation('delete domain', self):
             try:
                 self.power_off()
 
                 instance = self._get_domain()
-                instance.undefine()
+                if instance:
+                    instance.undefine()
             except Exception as e:
                 util.ignore_exception('instance delete', e)
 
-        with util.RecordedOperation('delete disks', self) as _:
+        with util.RecordedOperation('delete disks', self):
             try:
-                shutil.rmtree(self.instance_path)
+                if os.path.exists(self.instance_path):
+                    shutil.rmtree(self.instance_path)
             except Exception as e:
                 util.ignore_exception('instance delete', e)
 
-        with util.RecordedOperation('release network addresses', self) as _:
+        with util.RecordedOperation('release network addresses', self):
             for ni in db.get_instance_interfaces(self.db_entry['uuid']):
                 db.update_network_interface_state(ni['uuid'], 'deleted')
-                with db.get_lock('sf/ipmanager/%s' % ni['network_uuid'],
-                                 ttl=120) as _:
+                with db.get_lock('ipmanager', None, ni['network_uuid'],
+                                 ttl=120):
                     ipm = db.get_ipmanager(ni['network_uuid'])
                     ipm.release(ni['ipv4'])
                     db.persist_ipmanager(ni['network_uuid'], ipm.save())
 
-        db.update_instance_state(self.db_entry['uuid'], 'deleted')
-        db.update_instance_power_state(self.db_entry['uuid'], 'off')
         db.free_console_port(self.db_entry['console_port'])
         db.free_console_port(self.db_entry['vdi_port'])
 
@@ -302,7 +311,7 @@ class Instance(object):
             'hostname': '%s.local' % self.db_entry['name'],
             'launch_index': 0,
             'devices': [],
-            'project_id': self.tenant,
+            'project_id': None,
             'name': self.db_entry['name'],
             'public_keys': {
                 'mykey': self.db_entry['ssh_key']
@@ -445,10 +454,21 @@ class Instance(object):
             return conn.lookupByName('sf:' + self.db_entry['uuid'])
 
         except libvirt.libvirtError:
-            LOG.error('%s: Failed to lookup domain' % self)
             return None
 
+    def is_powered_on(self):
+        instance = self._get_domain()
+        if not instance:
+            return 'off'
+
+        libvirt = util.get_libvirt()
+        return util.extract_power_state(libvirt, instance)
+
     def power_on(self):
+        if not os.path.exists(self.xml_file):
+            db.enqueue_instance_error(self.db_entry['uuid'],
+                                      'missing domain file in power on')
+
         libvirt = util.get_libvirt()
         with open(self.xml_file) as f:
             xml = f.read()
@@ -458,13 +478,16 @@ class Instance(object):
             conn = libvirt.open(None)
             instance = conn.defineXML(xml)
             if not instance:
-                LOG.error('%s: Failed to create libvirt domain' % self)
-                return
+                db.enqueue_instance_error(self.db_entry['uuid'],
+                                          'power on failed to create domain')
+                raise exceptions.NoDomainException()
 
         try:
             instance.create()
-        except libvirt.libvirtError:
-            pass
+        except libvirt.libvirtError as e:
+            if not str(e).startswith('Requested operation is not valid: domain is already running'):
+                LOG.withObj(self).warning('Instance start error: %s' % e)
+                return False
 
         instance.setAutostart(1)
         db.update_instance_power_state(
@@ -472,30 +495,25 @@ class Instance(object):
             util.extract_power_state(libvirt, instance))
         db.add_event(
             'instance', self.db_entry['uuid'], 'poweron', 'complete', None, None)
+        return True
 
     def power_off(self):
         libvirt = util.get_libvirt()
-        with open(self.xml_file) as f:
-            xml = f.read()
-
         instance = self._get_domain()
         if not instance:
-            conn = libvirt.open(None)
-            instance = conn.defineXML(xml)
-            if not instance:
-                LOG.error('%s: Failed to create libvirt domain' % self)
-                return
+            return
 
         try:
             instance.destroy()
         except libvirt.libvirtError as e:
-            LOG.error('%s: Failed to delete domain: %s' % (self, e))
+            LOG.withObj(self).error('Failed to delete domain: %s' % e)
 
+        db.update_instance_power_state(self.db_entry['uuid'], 'off')
         db.add_event(
-            'instance', self.db_entry['uuid'], 'poweron', 'complete', None, None)
+            'instance', self.db_entry['uuid'], 'poweroff', 'complete', None, None)
 
     def _snapshot_device(self, source, destination):
-        images.snapshot(source, destination)
+        images.snapshot(None, source, destination)
 
     def snapshot(self, all=False):
         disks = self.db_entry['block_devices']['devices']
@@ -505,8 +523,8 @@ class Instance(object):
         snapshot_uuid = str(uuid.uuid4())
         snappath = os.path.join(self.snapshot_path, snapshot_uuid)
         if not os.path.exists(snappath):
-            LOG.debug('%s: Creating snapshot storage at %s' %
-                      (self, snappath))
+            LOG.withObj(self).debug(
+                'Creating snapshot storage at %s' % snappath)
             os.makedirs(snappath)
             with open(os.path.join(self.snapshot_path, 'index.html'), 'w') as f:
                 f.write('<html></html>')
@@ -521,7 +539,7 @@ class Instance(object):
             if d['type'] != 'qcow2':
                 continue
 
-            with util.RecordedOperation('snapshot %s' % d['device'], self) as _:
+            with util.RecordedOperation('snapshot %s' % d['device'], self):
                 self._snapshot_device(
                     d['path'], os.path.join(snappath, d['device']))
                 db.create_snapshot(snapshot_uuid, d['device'], self.db_entry['uuid'],
@@ -567,8 +585,15 @@ class Instance(object):
         if not os.path.exists(console_path):
             return ''
 
-        offset = max(0, os.stat(console_path).st_size - length)
+        d = None
+        file_length = os.stat(console_path).st_size
+        with open(console_path, 'rb') as f:
+            if length != -1:
+                offset = max(0, file_length - length)
+                f.seek(offset)
+            d = f.read()
 
-        with open(console_path) as f:
-            f.seek(offset)
-            return f.read()
+        LOG.withObj(self).info(
+            'Client requested %d bytes of console log, returning %d bytes'
+            % (length, len(d)))
+        return d
